<?xml version="1.0"?>

<!--
  Copyright (c) 1997-2010
  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
  http://www.polymake.de

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version: http://www.gnu.org/licenses/gpl.txt.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
===============================================================================
  $Id: upgrade-2.9.10.xslt 10071 2011-03-17 13:04:15Z herr $

  In all objects of type Polytope or derived thereof:
    Replace AMBIENT_DIM by POLYTOPE_AMBIENT_DIM 
    Replace DIM by POLYTOPE_DIM or COMBINATORIAL_DIM, depending on presence of geometric information
-->

<xsl:transform version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
			     xmlns:p="http://www.math.tu-berlin.de/polymake/#3">

<xsl:output method="xml" encoding="utf-8" cdata-section-elements="description" indent="yes" />

<!-- detect the Polytope objects: any top-level object of application polytope except few ones -->
<xsl:template match="/p:object[@type[starts-with(string(),'polytope::') and not(contains(string(),'PolyhedralFan'))]]">
  <xsl:element name="{name()}" namespace="{namespace-uri()}">
    <xsl:copy-of select="@*[name()!='tm' and name()!='version']" />
    <xsl:apply-templates select="./node()" mode="polytope" />
  </xsl:element>
</xsl:template>

<!-- ... or a property of matroid, point configuration, etc. -->
<xsl:template match="p:property[@name='POLYTOPE' or @name='CONVEX_HULL' or @name='ENVELOPE']/p:object">
  <xsl:element name="{name()}" namespace="{namespace-uri()}">
    <xsl:copy-of select="@*" />
    <xsl:apply-templates select="./node()" mode="polytope" />
  </xsl:element>
</xsl:template>

<!-- rename the dimension properties -->

<xsl:template match="p:property[@name='AMBIENT_DIM']" mode="polytope">
<xsl:variable name="ambidim" select="@value" /> 
  <xsl:element name="property" namespace="{namespace-uri()}">
    <xsl:attribute name="name">CONE_AMBIENT_DIM</xsl:attribute>
    <xsl:attribute name="value">
      <xsl:value-of select="$ambidim + 1" />
    </xsl:attribute>
  </xsl:element>
</xsl:template>

<xsl:template match="p:property[@name='DIM']" mode="polytope">
<xsl:variable name="dim" select="@value" /> 
  <xsl:choose>
    <xsl:when test="../p:property[@name='POINTS'] or ../p:property[@name='VERTICES'] or ../p:property[@name='INEQUALITIES'] or ../p:property[@name='FACETS'] or ../p:property[@name='EQUATIONS'] or ../p:property[@name='AFFINE_HULL'] or ../p:property[@name='LINEALITY_SPACE'] ">
      <xsl:element name="property" namespace="{namespace-uri()}">
        <xsl:attribute name="name">CONE_DIM</xsl:attribute>
        <xsl:attribute name="value">
          <xsl:value-of select="$dim + 1" />
        </xsl:attribute>
      </xsl:element>
    </xsl:when>
    <xsl:otherwise>
      <xsl:element name="property" namespace="{namespace-uri()}">
        <xsl:attribute name="name">COMBINATORIAL_DIM</xsl:attribute>
        <xsl:attribute name="value">
          <xsl:value-of select="@value" />
        </xsl:attribute>
      </xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- everything else is copied verbatim by switching back to the default mode -->

<xsl:template match="*" mode="polytope">
  <xsl:apply-templates select="." />
</xsl:template>

<xsl:include href="trivial-copy.xslt" />

</xsl:transform>
